(ns net.cgrand.enliven
  "Incubatory namespace for the next version of Enlive."
  #_(:require [net.cgrand.enlive-html :as h])
  (:require [net.cgrand.xmacros :as x]))

(defprotocol Transformation
  (selector [t])
  (data-selector [t])
  (tinvoke [t nodes data]))

(extend-protocol Transformation
  clojure.lang.Fn
  (selector [t] identity)
  (data-selector [t] identity)
  (tinvoke [f nodes data] (f nodes data)))



(defn )

(defn foo [src selectors-tfns]
  (let [selections-tfns
          (map (fn [[selector tfn]] [(zip-select src selector) tfns]) selectors-tfns)
        selections (set (keys selections-tfns))]))



(defprotocol Fragment
  (nodes [x] "Returns a seq of nodes"))

(extend-protocol Fragment
  String 
  (nodes [text] (list text))
  clojure.lang.APersistentMap
  (nodes [node] (list node)))

(defprotocol Transformation
  (invoke [fragment data]))

(def transform
  "Transformation whose data is a rules-maps or a fn as returned by #'rules."
  (reify Transformation
    (invoke [fragment rules]
      (if (fn? rules) 
        (rules fragment)
        XXX))))

;; Functions are considered transformations of one node, they are applied 
;; to each node passed to them.
(extend-protocol Transformation
  clojure.lang.Fn
  (invoke [f frag data]
    (reduce #(concat %1 (nodes (f %2 data))) (nodes frag))))

(defmacro tfn 
 "Defines a transformation -- only useful when you want to deal with 
  fragments as a whole."
  [args & body]
  `(reify Transformation
     (invoke ~args ~@body)))

(defn one-by-one 
  "Takes a transformation and returns a transformation which applies the
  original one to each node individually."
  [t]
  (fn [node] (t node)))

(defmacro rules
  "Expands to a map of selectors to fns or to a single function (for the 2 
  arity). The result can then be passed to transform."
  ([op data] 
    (x/expand-op op data))
  ([selector op data & more]
    (into {} (for [[sel op data] (partition 3 (list* selector op data more))]
               [sel (x/expand-op op data)]))))

(x/defdefault-op rules [tfn data]
  `(fn [frag#] (invoke ~tfn frag# ~data)))

(x/defop rules :for [[seq-exprs & body]]
  `(fn [frag#] 
     (apply concat (for ~seq-exprs
                     (nodes (invoke transform frag# (rules ~@body)))))))

(x/defop rules :do [body2]
  `(comp ~@(reverse (for [[op data] (partition 2 body2)]
                      (x/expand-macro-op rules op data)))))

(x/defop rules :sub
 "Performs a sub transformation."
 [body]
  `(fn [frag#] (invoke transform nodes# (rules ~@body))))

(defn set-attrs [element attrs]
  (assoc element :attrs (merge (:attrs element) attrs)))





(defn- split-options [options-and-stuff]
  (loop [s options-and-stuff opts {}]
    (let [[k v & more] s]
      (if (and (next s) (keyword? k))
        (recur more (assoc opts k v))
        [opts s]))))

(defmacro deftemplate 
  "Supported options: :doc :resource :only"
  [name resource docstring? & options-and-body]
  (let [[opts body] 
          (cond 
            (keyword? resource)
              (let [[opts body] (split-options options-and-body)]
                [(assoc opts resource docstring?) body])
            (string? docstring?)
              (let [[opts body] (split-options options-and-body)]
                [(assoc opts :resource resource :doc docstring?) body])
            :else
              (split-options (cons docstring? options-and-body)))
          bodies (if (vector? (first body)) [body] body)]
    ))



